/*
*
* Author: Giacomo Lozito <james@develia.org>, (C) 2005-2006
*
* This program is free software; you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the
* Free Software Foundation; either version 2 of the License, or (at your
* option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*
*/

#include "mon.h"

#ifdef USE_GAMIN

#include <fam.h>
#include <unistd.h>
#include <fcntl.h>



typedef struct
{
  gint (*func_deleted)( gpointer );
  gint (*func_created)( gpointer );
  gpointer arg;
}
gmu_MonitorFunc;


static FAMConnection *fam_c = NULL;
static GIOChannel *fam_giochannel = NULL;


gint
gmu_mon_init ( void )
{
  if ( fam_c == NULL )
  {
    fam_c = g_malloc0(sizeof(FAMConnection));
    if ( FAMOpen2( fam_c , "gmutoo" ) != 0 )
    {
      /* FAMOpen failed */
      DEBUGMSG( "FAMOpen failed! Monitor system aborted!\n" );
      g_free( fam_c );
      fam_c = NULL;
      return -1;
    }
    else
    {
      gint fam_fd = FAMCONNECTION_GETFD(fam_c);
      fcntl( fam_fd , F_SETFL , fcntl(fam_fd,F_GETFL) | O_NONBLOCK );
      fam_giochannel = g_io_channel_unix_new( fam_fd );
      g_io_add_watch( fam_giochannel , G_IO_IN , gmu_mon_event , NULL );
      g_io_channel_unref( fam_giochannel );
      DEBUGMSG( "FAMOpen ok! Monitor system started!\n" );
      return 0;
    }
  }
  else
  {
    /* multiple calls of gmu_mon_init should not be accepted */
    DEBUGMSG( "Monitor system has been already started! Returning a failure for this call.\n" );
    return -1;
  }
}


gint
gmu_mon_shutdown ( void )
{
  g_io_channel_unref( fam_giochannel );
  FAMClose( fam_c );
  DEBUGMSG( "FAMClose ok! Monitor system halted!\n" );
  return 0;
}


gint
gmu_mon_add_file ( gchar * filename ,
                   gpointer * fam_rq_returnptr ,
                   gint (*f_del)( gpointer ) ,
                   gint (*f_cre)( gpointer ) ,
                   gpointer argument )
{
  FAMRequest *fam_rq = g_malloc0(sizeof(FAMRequest));
  gmu_MonitorFunc *mf = g_malloc(sizeof(gmu_MonitorFunc));
  mf->func_deleted = f_del;
  mf->func_created = f_cre;
  mf->arg = argument;
  if ( FAMMonitorFile( fam_c , filename , fam_rq , mf ) != 0 )
  {
    /* FAMMonitorFile failed */
    DEBUGMSG( "FAMMonitorFile failed! Monitor for '%s' aborted.\n" , filename );
    g_free( fam_rq );
    *fam_rq_returnptr = NULL;
    return -1;
  }
  else
  {
    DEBUGMSG( "FAMMonitorFile ok! Monitor for '%s' started.\n" , filename );
    *fam_rq_returnptr = fam_rq;
    return 0;
  }
}


gint
gmu_mon_remove_file ( gpointer fam_rq_ptr )
{
  FAMRequest *fam_rq = fam_rq_ptr;
  if ( FAMCancelMonitor( fam_c , fam_rq ) != 0 )
  {
    g_warning( "An error occurred after calling FAMCancelMonitor.\n" );
    return -1;
  }
  else
  {
    DEBUGMSG( "FAMCancelMonitor ok!\n" );
    return 0;
  }
}


gboolean
gmu_mon_event ( GIOChannel * source , GIOCondition condition , gpointer data )
{
  gint result = FAMPending( fam_c );

  while ( result != 0 )
  {
    if ( result == 1 )
    {
      /* FAM event queued */
      FAMEvent fam_ev;
      if ( FAMNextEvent( fam_c , &fam_ev ) != 1 )
      {
        g_warning( "Error while recovering the next FAM event.\n" );
        return TRUE;
      }
      else
      {
        switch( fam_ev.code )
        {
          case FAMChanged: /* do nothing */
            DEBUGMSG( "FAMChanged event for file '%s'.\n" , fam_ev.filename );
            break;

          case FAMDeleted: /* change information */
          {
            gmu_MonitorFunc *mf = fam_ev.userdata;
            mf->func_deleted( mf->arg );
            DEBUGMSG( "FAMDeleted event for file '%s'.\n" , fam_ev.filename );
            break;
          }

          case FAMStartExecuting: /* do nothing */
            DEBUGMSG( "FAMStartExecuting event for file '%s'.\n" , fam_ev.filename );
            break;

          case FAMStopExecuting: /* do nothing */
            DEBUGMSG( "FAMStopExecuting event for file '%s'.\n" , fam_ev.filename );
            break;

          case FAMCreated: /* change information */
          {
            gmu_MonitorFunc *mf = fam_ev.userdata;
            mf->func_created( mf->arg );
            DEBUGMSG( "FAMCreated event for file '%s'.\n" , fam_ev.filename );
            break;
          }

          case FAMAcknowledge: /* cleanup routine */
          {
            DEBUGMSG( "FAMAcknowledge event for file '%s'.\n" , fam_ev.filename );
            /* free the gmu_MonitorFunc structure */
            g_free( fam_ev.userdata );
            break;
          }

          case FAMExists: /* change information */
          {
            gmu_MonitorFunc *mf = fam_ev.userdata;
            mf->func_created( mf->arg );
            DEBUGMSG( "FAMExists event for file '%s'.\n" , fam_ev.filename );
            break;
          }

          case FAMEndExist: /* do nothing */
            DEBUGMSG( "FAMEndExist event for file '%s'.\n" , fam_ev.filename );
            break;

          default: /* something else? */
            DEBUGMSG( "Other event (?) for file '%s'.\n" , fam_ev.filename );
            break;
        }
      }
    }
    else
    {
      /* an error occurred */
      g_warning( "An error occurred in FAMPeding (monitoring system).\n" );
      return TRUE;
    }

    /* check if there's another event */
    result = FAMPending( fam_c );
  }
  return TRUE;
}

#else  /* use empty functions if USE_GAMIN is not defined */

gint
gmu_mon_init ( void )
{
  return 0;
}

gint
gmu_mon_shutdown ( void )
{
  return 0;
}

gint
gmu_mon_add_file ( gchar * filename ,
                   gpointer * fam_rq_returnptr ,
                   gint (*f_del)( gpointer ) ,
                   gint (*f_cre)( gpointer ) ,
                   gpointer argument )
{
  return 0;
}

gint
gmu_mon_remove_file ( gpointer p )
{
  return 0;
}

gboolean
gmu_mon_event ( GIOChannel * ch , GIOCondition co , gpointer p )
{
  return TRUE;
}

#endif /* USE_GAMIN */
